<?php

namespace Drp\AdfsAuthentication;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;

class AdfsOAuthToken extends Model
{
    /**
     * @var string
     */
    protected $table = 'adfs_oauth_tokens';

    /**
     * @var array
     */
    protected $fillable = [
        'user_id', 'access_token', 'access_token_expiry', 'refresh_token', 'refresh_token_expiry', 'id_token'
    ];

    /**
     * @var string
     */
    protected $primaryKey = 'access_token';

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasOne
     */
    public function user()
    {
        return $this->hasOne(\App\User::class);
    }

    /**
     * @param \Illuminate\Database\Eloquent\Builder $query
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function scopeExpiringSoon(Builder $query): Builder
    {
        return $query->where('access_token_expiry', '<', Carbon::now()->addMinutes(10));
    }

    /**
     * @param string $token
     * @param integer $expiry
     * @return void
     */
    public function updateAccessToken($token, $expiry)
    {
        $this->access_token = $token;
        $this->access_token_expiry = $expiry;
        $this->save();
    }

    /**
     * @param string $token
     * @param integer $expiry
     * @return void
     */
    public function updateRefreshToken($token, $expiry)
    {
        $this->refresh_token = $token;
        $this->refresh_token_expiry = $expiry;
        $this->save();
    }
}
