<?php

namespace Drp\AdfsAuthentication;

use Drp\AdfsAuthentication\Commands\RefreshExpiringTokens;
use Illuminate\Foundation\Support\Providers\AuthServiceProvider as ServiceProvider;
use Illuminate\Support\Facades\Auth;
use Laravel\Passport\Bridge\UserRepository;
use Laravel\Passport\Passport;

class AdfsServiceProvider extends ServiceProvider
{
    /**
     * The policy mappings for the application.
     *
     * @var array
     */
    protected $policies = [
        'App\Model' => 'App\Policies\ModelPolicy',
    ];

    /**
     * Register ADFS Services
     *
     * @return void
     */
    public function boot()
    {
        $this->registerPolicies();

        /**
         * Load the custom migrations for the adfs_users
         * and adfs_oauth_tokens tables
         */
        $this->loadMigrationsFrom(__DIR__ . '/database/migrations');

        /**
         * Publish the custom configuration files
         */
        $this->publishes([
            __DIR__ . '/../config/adfs.php' => config_path('adfs.php'),
        ]);

        /**
         * Setup the new ADFS user provider.
         */
        Auth::provider('adfsuserprovider', function ($app, array $config) {
            return new AdfsUserProvider($app['hash'], $config['model'], app(AdfsHandler::class));
        });

        /**
         * Override the Passport UserRepository for Auth.
         */
        $this->app->bind(UserRepository::class, function () {
            return new AdfsUserRepository(Auth::createUserProvider(config('auth.guards.api.provider')));
        });

        /**
         * Register the command for refreshing
         * refresh and access tokens
         */
        $this->commands([RefreshExpiringTokens::class]);

        /**
         * Register the Passport OAuth Routes
         */
        Passport::routes();
    }

    /**
     * Get the services provided by the provider.
     * @return array
     */
    public function provides(): array
    {
        return ['auth'];
    }

    /**
     * Returns the root Auth instance.
     * @return mixed
     */
    protected function getAuth()
    {
        return Auth::getFacadeRoot();
    }
}
