<?php

namespace Drp\AdfsAuthentication;

use Exception;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;
use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\OAuth2\Server\Exception\OAuthServerException;
use League\OAuth2\Server\Repositories\UserRepositoryInterface;

class AdfsUserRepository implements UserRepositoryInterface
{

    /**
     * @var AdfsUserProvider
     */
    protected $provider;

    /**
     * AdfsUserRepository constructor.
     * @param AdfsUserProvider $provider
     */
    public function __construct(AdfsUserProvider $provider)
    {
        $this->provider = $provider;
    }

    /**
     * @param string $username
     * @param string $password
     * @param string $grantType
     * @param ClientEntityInterface $clientEntity
     * @throws OAuthServerException
     * @return \Illuminate\Contracts\Auth\Authenticatable|null
     * @SuppressWarnings(PHPMD)
     */
    public function getUserEntityByUserCredentials(
        $username,
        $password,
        $grantType,
        ClientEntityInterface $clientEntity
    ) {
        /**
         * Called by passport to validate the credentials.
         * Overridden to connect to ADFS instead of the DB.
         */
        try {
            return $this->provider->retrieveByCredentials(['username' => $username, 'password' => $password]);
        } catch (ClientException $e) {
            throw OAuthServerException::invalidCredentials();
        } catch (ServerException $e) {
            throw OAuthServerException::serverError(
                $e->getResponse()->getStatusCode() . ': ' . $e->getResponse()->getReasonPhrase()
            );
        } catch (Exception $e) {
            throw OAuthServerException::serverError($e->getMessage());
        }
    }
}
